# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the metadata
label on the player.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Manager(object):
	"""
	This class defines the behavior of the metadata label on the player.
	"""

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player.
		@type player: A Player object.
		"""
		self.__init_attributes(player)
		self.__update_label()
		self.__signal_id_1 = player.connect("stream-update", self.__stream_update_cb)
		self.__signal_id_2 = player.connect("streamer-is-busy", self.__streamer_is_busy_cb)
		self.__signal_id_3 = player.connect("streamer-is-stopped", self.__streamer_is_stopped_cb)
		self.__signal_id_4 = player.connect("playlist-error", self.__playlist_error_cb)
		self.__signal_id_5 = player.connect("stream-error", self.__playlist_error_cb)

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		self.__label = player.glade.get_widget("MetadataLabel")
		self.__string = "<b><span foreground='blue'>%s</span></b>\n<i>by</i> %s <span weight='ultralight' size='small' foreground='brown'>%s</span>"
		self.__inactive_string = "<b>%s</b>\n<i>by</i> %s <span weight='ultralight' size='small'>0min</span>"
		self.__signal_id_1 = self.__signal_id_2 = None
		self.__signal_id_3 = self.__signal_id_4 = None
		self.__signal_id_5 = None
		return

	def __update_label(self):
		"""
		Update the label at startup.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		if self.__player.is_playing:
			title = self.__player.title
			station = self.__player.station
			time = self.__player.time
			self.__metadata_message(title, station, time)
		elif self.__player.is_busy:
			self.__info_message()
		return

	def __message(self, title, station):
		"""
		Update the label without any markup.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param title: The title of the song playing or informational message.
		@type title: A String object.

		@param station: The name of the radio station or informational message.
		@type station: A String object.
		"""
		self.__label.set_markup(self.__inactive_string % (title.strip(), station.strip()))
		self.__label.set_property("sensitive", False)
		return

	def __default_message(self):
		"""
		Default label message.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		title = "Artist - Title"
		station = "Radio Station"
		self.__message(title, station)
		return

	def __info_message(self):
		"""
		Informational label message.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.i18n import msg0001
		self.__message(msg0001, msg0001)
		return

	def __metadata_message(self, title, station, time):
		"""
		Informational label message about radio station and song.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param title: Title of song playing.
		@type title: A String object.

		@param station: Name of radio station.
		@type station: A String object.

		@param time: How long the radio station has been playing.
		@type time: A String object.
		"""
		self.__label.set_markup(self.__string % (title, station, time))
		self.__label.set_property("sensitive", True)
		return

	def __stream_update_cb(self, player, dictionary):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param dictionary: A dictionary containing metadata information.
		@type dictionary: A Dictionary object.
		"""
		try:
			title = dictionary["title"].strip()
			station = dictionary["station"].strip()
			time = dictionary["time"]
			self.__metadata_message(title, station, time)
		except KeyError:
			pass
		return

	def __streamer_is_busy_cb(self, *args):
		"""
		Handles callback when the "streamer-is-busy" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__info_message()
		return

	def __streamer_is_stopped_cb(self, *args):
		"""
		Handles callback when the "streamer-is-stopped" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__default_message()
		return

	def __playlist_error_cb(self, *args):
		"""
		Handles callback when the "playlist-error" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__default_message()
		return

	def destroy(self):
		"""
		Destroy object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__player)
		disconnect_signal(self.__signal_id_2, self.__player)
		disconnect_signal(self.__signal_id_3, self.__player)
		disconnect_signal(self.__signal_id_4, self.__player)
		disconnect_signal(self.__signal_id_5, self.__player)
		self.__label.destroy()
		delete_attributes(self)
		del self
		self = None
		return
