# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that parsers Shoutcast playlists and
updates the information in a database.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_STRING, TYPE_PYOBJECT

class Manager(GObject):
	"""
	This class parsers Shoutcast playlist files. The information in the
	Shoutcast file is stored in two databases; stations.gdb and
	playlist.gdb. stations.gdb contains a list of all stations that
	can be played. playlist.gdb contains only one entry, the station
	currently playing.
	"""

	__gsignals__ = {
		"parsed": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"parse-error": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
	}

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		GObject.__init__(self)
		self.__init_attributes(player)
		self.__sig_id_1 = player.connect("parse-list", self.__parse_list_cb)
		self.__sig_id_2 = self.connect("parsed", self.__parsed_cb)
		self.__sig_id_3 = self.connect("parse-error", self.__parser_error_cb)
		player.emit("parser-is-ready")

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		self.__sig_id_1 = self.__sig_id_2 = self.__sig_id_3 = None
		self.__valid_pls_files = set([])
		self.__invalid_files = set([])
		# Format: {station_name: user_defined_name, genre, set([uri1, uri2, uri3, ...])}
		self.__stations_dictionary = {}
		return

########################################################################
#
#							Public API
#
########################################################################

	def destroy(self):
		"""
		Destroy object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__sig_id_1, self.__player)
		disconnect_signal(self.__sig_id_2, self)
		disconnect_signal(self.__sig_id_3, self)
		delete_attributes(self)
		del self
		self = None
		return

########################################################################
#
#							Helper Methods
#
########################################################################

	def __update_database(self):
		"""
		Update stations.gdb and playlist.gdb.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		# Update stations database.
		from operator import not_
		if not_(self.__stations_dictionary): return self.__reset()
		from Metadata import update_stations_database, update_playlist_database
		from Metadata import get_stations_from_database
		stations = get_stations_from_database()
		if not_(stations): stations = self.__stations_dictionary
		stations.update(self.__stations_dictionary)
		update_stations_database(stations)
		# Update playlist database.
		dictionary = {}
		for station_name, data in self.__stations_dictionary.iteritems():
			dictionary[station_name] = data
			update_playlist_database(dictionary)
			break
		self.__reset()
		return

	def __error(self):
		"""
		Show error messages.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		files = ", ".join(self.__invalid_files)
		print "Error:", files, "invalid files."
		self.__reset()
		return

	def __reset(self):
		self.__valid_pls_files.clear()
		self.__invalid_files.clear()
		self.__stations_dictionary.clear()
		return

########################################################################
#
#						Event and Signal Handlers
#
########################################################################

	def __parse_list_cb(self, player, data):
		"""
		Handles callback when the "parse-list" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.

		@param plslist: A list of shoutcast playlist files to parse.
		@type plslist: A List object.
		"""
		self.__reset()
		player.emit("parser-is-busy")
		from operator import not_
		plslist = set(data)
		is_not_pls = lambda x: not_(x.endswith(".pls"))
		self.__invalid_files = set(filter(is_not_pls, plslist))
		self.__valid_pls_files = plslist.difference(set(self.__invalid_files))
		from Parser import Parser
		for playlist_file in self.__valid_pls_files:
			Parser(self, playlist_file)
		if self.__valid_pls_files: return
		self.__update_database()
		if self.__invalid_files: self.__error()
		self.__player.emit("finished-parsing")
		return

	def __parsed_cb(self, manager, data):
		"""
		Handles callback when the "parsed" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param data: playlist file and station dictionary.
		@type data: A Tuple object.
		"""
		playlist_file, station_dictionary = data
		self.__stations_dictionary.update(station_dictionary)
		self.__valid_pls_files.remove(playlist_file)
		if self.__valid_pls_files: return
		self.__update_database()
		if self.__invalid_files: self.__error()
		self.__player.emit("finished-parsing")
		return

	def __parser_error_cb(self, manager, data):
		"""
		Handles callback when the "parse-error" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param data: Playlist file.
		@type data: A String object.
		"""
		playlist_file = data
		self.__valid_pls_files.remove(playlist_file)
		self.__invalid_files.add(playlist_file)
		if self.__valid_pls_files: return
		self.__update_database()
		if self.__invalid_files: self.__error()
		self.__player.emit("finished-parsing")
		return
