# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that streams music data from internet
radio stations.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Manager(object):
	"""
	This class streams music data from internet radio stations.
	"""

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(player)
		self.__set_properties()
		self.__bus.add_signal_watch()
		from gobject import idle_add
		idle_add(self.__precompile_methods)
		player.connect("toggle-play", self.__toggle_play_cb)
		self.__bus.connect("message::tag", self.__message_tag_cb)
		self.__bus.connect("message::error", self.__message_error_cb)
		self.__bus.connect("message::info", self.__message_info_cb)
		self.__bus.connect("message::warning", self.__message_warning_cb)
		self.__bus.connect("message::buffering", self.__message_buffering_cb)
		player.connect("streamer-is-busy", self.__streamer_is_busy_cb)
		player.connect("streamer-is-stopped", self.__streamer_is_stopped_cb)
		player.connect("stream-update", self.__stream_update_cb)
		player.connect("stream-error", self.__stream_error_cb)
		player.connect("stream-message", self.__stream_message_cb)
		player.connect("playlist-error", self.__playlist_error_cb)
		player.connect("volume", self.__volume_cb)
		from gnomevfs import monitor_add, MONITOR_FILE
		self.__monitor_id = monitor_add(self.__database_uri, MONITOR_FILE,
					self.__database_changed_cb)
		player.emit("streamer-is-ready")
		player.emit("volume-changed", self.__element.get_property("volume"))
		self.__reset()
		player.response()

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__response = False
		from gst import element_factory_make
		# Path to the abbreviation database.
		from Striim.Global import metadata_folder
		self.__database_path = metadata_folder + "/playlist.gdb"
		from gnomevfs import get_uri_from_local_path
		self.__database_uri = get_uri_from_local_path(self.__database_path)
		self.__element = element_factory_make("playbin", "MainPipeline")
		self.__bus = self.__element.get_bus()
		self.__title = None
		self.__uris = set([])
		self.__uri = None
		self.__timeout = None
		self.__station = None
		self.__is_playing = False
		self.__metadata_dictionary = {}
		self.__player = player
		self.__feedback = player.feedback
		self.__play_time = None
		self.__can_pause = True
		self.__status_id = None
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		volume = self.__get_volume_from_database()
		self.__element.set_property("volume", volume)
		queue_threshold = self.__element.get_property("queue-threshold") * 10
		queue_size = self.__element.get_property("queue-size") * 1000000
		self.__element.set_property("queue-threshold", long(queue_threshold))
		self.__element.set_property("queue-size", long(queue_size))
		return

	def __get_volume_from_database(self):
		from Metadata import get_volume_from_database
		volume = get_volume_from_database()
		if volume: return volume
		return 1.5

########################################################################
#
#							Public API
#
########################################################################

	def destroy(self):
		"""
		Destroy object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		delete_attributes(self)
		del self
		self = None
		return

########################################################################
#
#						Helper Methods
#
########################################################################

	def __play(self):
		"""
		Stream radio station and play track.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from operator import not_
		if not_(self.__uri): return
		self.__player.emit("streamer-is-busy")
		self.__player.response()
		from gst import STATE_PLAYING
		self.__element.set_state(STATE_PLAYING)
		self.__is_playing = True
		self.__response = False
		from time import time
		self.__play_time = time()
		self.__player.response()
		return False

	def __pause(self):
		"""
		Stop streaming radio station.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__stop()
		self.__player.emit("streamer-is-stopped")
		self.__player.response()
		return False

	def __stop(self):
		self.__player.response()
		from gst import STATE_NULL
		self.__element.set_state(STATE_NULL)
		self.__is_playing = False
		self.__title = None
		self.__response = True
		self.__metadata_dictionary.clear()
		self.__player.response()
		return False

	def __replay(self):
		"""
		Restream radio station.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__stop()
		self.__set_uri()
		from gobject import idle_add
		idle_add(self.__play)
		return False

	def __reset(self):
		"""
		Reset radio station and stream.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		try:
			from Exceptions import PlaylistError
			self.__get_playlist_from_database()
			from gobject import idle_add
			idle_add(self.__replay)
		except PlaylistError:
			self.__player.emit("playlist-error")
		return False

	def __calculate_time(self):
		"""
		Calculate how long the a radio station has been streaming.

		There's got to be a better way to do this, via GStreamer.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param time: How long a station has been playing in hours and minutes.
		@type time: A String object.
		"""
		from time import time
		self.__current_time = time()
		duration = int((self.__current_time - self.__play_time) / 60)
		if duration == 0:
			time = "1min"
		elif duration >= 60:
			hour = str(int(duration / 60))
			mins = str(int(duration % 60))
			if mins == "0":
				time = "%shr" % (hour)
			else:
				time = "%shr %smin" % (hour, mins)
		else:
			time = "%smin" % duration
		return time

	def __update_metadata_dictionary(self, message):
		"""
		Update metadata information about a striim.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param message: A message about a striim.
		@type message: A Structure object.
		"""
		self.__player.response()
		taglist = message.parse_tag()
		from operator import truth, not_, contains, eq
		if truth(taglist.is_empty()): return False
		if not_(contains(taglist.keys(), "title")): return False
		if eq(taglist["title"], self.__title): return False
		self.__title = taglist["title"]
		from xml.sax.saxutils import escape
		self.__metadata_dictionary["station"] = escape(self.__station)
		self.__metadata_dictionary["time"] = self.__calculate_time()
		self.__metadata_dictionary["title"] = escape(self.__title.strip())
		self.__player.emit("stream-update", self.__metadata_dictionary)
		self.__player.response()
		return False

	def __get_playlist_from_database(self):
		"""
		Get radio station to stream from database.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__stop()
		from Metadata import get_playlist_from_database
		dictionary = get_playlist_from_database()
		from operator import not_
		from Exceptions import PlaylistError
		if not_(dictionary): raise PlaylistError
		station_name = dictionary.keys()[0]
		user_defined_station_name, genre, self.__uris = dictionary.values()[0]
		if user_defined_station_name:
			self.__station = user_defined_station_name
		else:
			self.__station = station_name
		return

	def __set_uri(self):
		"""
		Set stream source.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from operator import not_
		if not_(self.__uris): return
		self.__uri = self.__uris.pop()
		self.__element.set_property("uri", self.__uri)
		return

	def __precompile_methods(self):
		"""
		Use psyco to precompile some methods to improve performance.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		try:
			from psyco import bind
			bind(self.__play)
			bind(self.__pause)
			bind(self.__update_metadata_dictionary)
			bind(self.__get_playlist_from_database)
			bind(self.__set_uri)
			bind(self.__calculate_time)
		except ImportError:
			pass
		return False

	def __responsiveness(self):
		"""
		Don't lock up the GUI when fetching metadata information.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@return: True to keep calling this function.
		@rtype: A Boolean object.
		"""
		self.__player.response()
		return self.__response

	def __connection_timeout(self):
		self.__uri = None
		self.__uris.clear()
		self.__stop()
		message = "No internet connection found"
		self.__player.emit("stream-error", message)
		return False

	def __remove_timeout(self):
		try:
			from gobject import source_remove
			source_remove(self.__timeout)
		except:
			pass
		return

########################################################################
#
#						Event and Signal Handlers
#
########################################################################

	def __toggle_play_cb(self, *args):
		"""
		Start or stop streaming radio station.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Exceptions import PlaylistError
		from gobject import idle_add
		if self.__is_playing:
			idle_add(self.__pause)
		else:
			idle_add(self.__reset)
		return

	def __message_tag_cb(self, bus, message):
		"""
		Handles callback when metadata information about a track is
		found.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param bus: A message system for a GStreamer element.
		@type bus: A GStreamer object.

		@param message: Some data.
		@type message: A data structure object.
		"""
		self.__remove_timeout()
		self.__player.response()
		from gobject import idle_add
		idle_add(self.__update_metadata_dictionary, message)
		#print self.__element.get_property("stream-info-value-array")
		self.__player.response()
		return True

	def __message_error_cb(self, bus, message):
		"""
		Handles callback when a stream error occurs.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param bus: A message system for a GStreamer element.
		@type bus: A GStreamer object.

		@param message: Some data.
		@type message: A data structure object.
		"""
		self.__player.response()
		error, debug = message.parse_error()
		#print debug
		if self.__uris:
			from gobject import idle_add
			idle_add(self.__replay)
		else:
			from xml.sax.saxutils import escape
			self.__remove_timeout()
			self.__player.emit("stream-error", escape(str(error)))
			#print error
		self.__player.response()
		return True

	def __message_buffering_cb(self, bus, message):
		"""
		Handles callback when a stream is buffering.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param bus: A message system for a GStreamer element.
		@type bus: A GStreamer object.

		@param message: Some data.
		@type message: A data structure object.
		"""
		self.__remove_timeout()
		self.__response = False
		self.__player.response()
		percentage = message.structure["buffer-percent"]
		string = "%s%% - buffering please wait..." % percentage
		from xml.sax.saxutils import escape
		self.__player.emit("stream-message", escape(string))
		self.__player.response()
		return True

	def __message_warning_cb(self, bus, message):
		"""
		Handles callback when a warning occurs.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param bus: A message system for a GStreamer element.
		@type bus: A GStreamer object.

		@param message: Some data.
		@type message: A data structure object.
		"""
		self.__remove_timeout()
		self.__player.response()
		warning = message.parse_warning()
		print warning
		self.__player.response()
		return True

	def __message_info_cb(self, bus, message):
		"""
		Handles callback when a info occurs.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param bus: A message system for a GStreamer element.
		@type bus: A GStreamer object.

		@param message: Some data.
		@type message: A data structure object.
		"""
		self.__remove_timeout()
		self.__player.response()
		info = message.parse_info()
		print info
		self.__player.response()
		return True

	def __playlist_error_cb(self, *args):
		"""
		Handles callback when the "playlist-error" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__remove_timeout()
		self.__response = False
		self.__is_playing = False
		if self.__status_id: self.__feedback.unset_modal_message(self.__status_id)
		message = "No radio station found"
		self.__status_id = self.__feedback.set_modal_message(message, "warning")
		return

	def __streamer_is_busy_cb(self, *args):
		"""
		Handles callback when the "streamer-is-busy" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__is_playing = False
		self.__response = True
		self.__feedback.unset_modal_message(self.__status_id)
		message = "Getting information..."
		if self.__status_id: self.__feedback.unset_modal_message(self.__status_id)
		self.__status_id = self.__feedback.set_modal_message(message, "connect")
		from gobject import timeout_add, PRIORITY_LOW
		self.__remove_timeout()
		self.__timeout = timeout_add(120000, self.__connection_timeout, priority=PRIORITY_LOW)
		timeout_add(250, self.__responsiveness, priority=PRIORITY_LOW)
		return

	def __streamer_is_stopped_cb(self, *args):
		"""
		Handles callback when the "streamer-is-stopped" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__is_playing = False
		self.__response = False
		if self.__status_id: self.__feedback.unset_modal_message(self.__status_id)
		self.__status_id = None
		return

	def __stream_update_cb(self, player, data):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.

		@param data: A dictionary containing metadata information.
		@type data: A Dictionary object.
		"""
		self.__response = False
		self.__is_playing = True
		message = "<i>%s</i>" % data["title"]
		if self.__status_id: self.__feedback.unset_modal_message(self.__status_id)
		self.__status_id = self.__feedback.set_modal_message(message, "info")
		return

	def __stream_message_cb(self, player, data):
		"""
		Handles callback when the "stream-message" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param data: A buffer message.
		@type data: A String object.
		"""
		self.__response = False
		self.__player.response()
		message = "<span color='brown'>%s</span>" % data
		self.__feedback.update_status_message(message, "run", 10)
		self.__player.response()
		return

	def __stream_error_cb(self, player, data):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.

		@param data: An error message.
		@type data: A String object.
		"""
		self.__response = False
		self.__is_playing = False
		data = "Error! " + data.strip(".")
		message = "<b><span color='red'>%s</span></b>" % data
		if self.__status_id: self.__feedback.unset_modal_message(self.__status_id)
		self.__status_id = self.__feedback.set_modal_message(message, "error")
		self.__remove_timeout()
		return

	def __database_changed_cb(self, monitor_uri, info_uri, event_type):
		"""
		Handles callback when the playlist database is modified.

		@param self: Reference to the Manager instance.
		@type self: An Manager object.

		@param monitor_uri: The uri that is monitored.
		@type monitor_uri: A String object.

		@param info_uri: The uri that is monitored.
		@type info_uri: A String object.

		@param event_type: The type of modification that occured.
		@type event_type: A gnomevfs.MONITOR_EVENT* object.
		"""
		from gnomevfs import MONITOR_EVENT_DELETED
		from gnomevfs import MONITOR_EVENT_CREATED
		from gnomevfs import MONITOR_EVENT_CHANGED
		if event_type in [MONITOR_EVENT_DELETED, MONITOR_EVENT_CREATED, MONITOR_EVENT_CHANGED]:
			self.__player.response()
			self.__reset()
			self.__player.response()
		return

	def __volume_cb(self, player, volume):
		self.__element.set_property("volume", volume)
		self.__player.emit("volume-changed", volume)
		self.__update_volume_database()
		return

	def __update_volume_database(self):
		from Metadata import update_volume_database
		update_volume_database(self.__element.get_property("volume"))
		return
